/*
 * PhysicsTest
 *
 * Written by Soothsayer in January 2010
 *
 * Website: http://www.cppanddelphi.atw.hu
 * Contact: cppanddelphi@gmail.com
 */
unit TVector;

interface

type
  CVector = record
    X, Y: Single;
  end;

  function CreateV(const AX, AY: Single): CVector;
  function AbsV(const V: CVector): CVector;
  function AddV(const A, B: CVector): CVector;
  function SubV(const A, B: CVector): CVector;
  function MultiplyV(const V: CVector; const Scalar: Single): CVector;
  function NegativeV(const V: CVector): CVector;
  function DotV(const V1, V2: CVector): Single;
  function Cross(const A, B: CVector): Single; overload;
  function Cross(const A: CVector; const S: Single): CVector; overload;
  function Cross(const S: Single; const A: CVector): CVector; overload;

implementation

function CreateV(const AX, AY: Single): CVector;
begin
  Result.X := AX;
  Result.Y := AY;
end;

function AbsV(const V: CVector): CVector;
begin
  Result.X := Abs(V.X);
  Result.Y := Abs(V.Y);
end;

function AddV(const A, B: CVector): CVector;
begin
  Result := CreateV(A.X + B.X, A.Y + B.Y);
end;

function SubV(const A, B: CVector): CVector;
begin
  Result := CreateV(A.X - B.X, A.Y - B.Y);
end;

function MultiplyV(const V: CVector; const Scalar: Single): CVector;
begin
  Result := CreateV(V.X * Scalar,V.Y * Scalar);
end;

function NegativeV(const V: CVector): CVector;
begin
  Result := CreateV(-V.X,-V.Y);
end;

function DotV(const V1, V2: CVector): Single;
begin
  Result := (V1.X * V2.X) + (V1.Y * V2.Y);
end;

function Cross(const A, B: CVector): Single; overload;
begin
  Result := (A.X * B.Y) - (A.Y * B.X);
end;

function Cross(const A: CVector; const S: Single): CVector; overload;
begin
  Result.X :=  S * A.Y;
  Result.Y := -S * A.X;
end;

function Cross(const S: Single; const A: CVector): CVector; overload;
begin
  Result.X := -S * A.Y;
  Result.Y :=  S * A.X;
end;

end.
