/*
 * PhysicsTest
 *
 * Written by Soothsayer in January 2010
 *
 * Website: http://www.cppanddelphi.atw.hu
 * Contact: cppanddelphi@gmail.com
 */
unit _display;

interface

uses
  Windows;
const
  {$IFDEF VER90}
    hSection = nil;
  {$ELSE}
    hSection = 0;
  {$ENDIF}
type
  TBitmapInfo = record
    bmiHeader: TBitmapInfoHeader;
    bmiColors: array[0..255] of RGBQUAD;
  end;
//-------------------------------------------------------------------------------------------------------
// Class display
//-------------------------------------------------------------------------------------------------------
  CDisplay = class

  private
    m_hBitmap:            HBITMAP;
    m_hBitmapDC:          HDC;
    m_nFrequency:         Int64;
    m_nStartTime:         Int64;
    m_nActualTime:        Int64;
    m_nBits:              Pointer;
    bmInfo:               TBitmapInfo;
  public
    Width:                Integer;
    Height:               Integer;

    constructor Create;
    destructor Destroy;
    procedure Init;
    procedure Clear(R, G, B: Byte);
    procedure DrawLine(X1, Y1, X2, Y2, Width: Integer; R, G, B: Byte);
    procedure Draw(WindowHandle: HWND);
  end;

implementation

//-------------------------------------------------------------------------------------------------------
// Creates display class
//-------------------------------------------------------------------------------------------------------
constructor CDisplay.Create;
var
  i, j, k: Integer;
begin
  Width := 640;
  Height := 400;
  m_hBitmapDC := CreateCompatibleDC(0);

  ZeroMemory(@bmInfo, sizeof(TBitmapInfo));
  bmInfo.bmiHeader.biSize := SizeOf(BITMAPINFOHEADER);
  bmInfo.bmiHeader.biWidth := Width;
  bmInfo.bmiHeader.biHeight := -Height;
  bmInfo.bmiHeader.biPlanes := 1;
  bmInfo.bmiHeader.biBitCount := 8;
  bmInfo.bmiHeader.biCompression := BI_RGB;

  for k := 0 to 5 do
  begin
    for j := 0 to 5 do
    begin
      for i := 0 to 5 do
      begin
        bmInfo.bmiColors[k * 36 + j * 6 + i].rgbRed := i * 51;
        bmInfo.bmiColors[k * 36 + j * 6 + i].rgbGreen := j * 51;
        bmInfo.bmiColors[k * 36 + j * 6 + i].rgbBlue := k * 51;
      end;
    end;
  end;
end;

//-------------------------------------------------------------------------------------------------------
// Destroys display class
//-------------------------------------------------------------------------------------------------------
destructor CDisplay.Destroy;
begin
  if m_hBitmapDC <> 0 then DeleteDC(m_hBitmapDC);
  if m_hBitmap <> 0 then DeleteObject(m_hBitmap);
end;

//-------------------------------------------------------------------------------------------------------
// Inits bitmap
//-------------------------------------------------------------------------------------------------------
procedure CDisplay.Init;
begin
  m_hBitmap := CreateDIBSection(0, PBitmapInfo(@bmInfo)^, DIB_RGB_COLORS, m_nBits, hSection, 0);
  DeleteObject(SelectObject(m_hBitmapDC, m_hBitmap));
  QueryPerformanceFrequency(m_nFrequency);
  QueryPerformanceCounter(m_nStartTime);
end;

//-------------------------------------------------------------------------------------------------------
// Clears the bitmap
//-------------------------------------------------------------------------------------------------------
procedure CDisplay.Clear(R, G, B: Byte);
var
  OldPen, NewPen: HPEN;
  OldBrush, NewBrush: HBRUSH;
begin
  NewPen := CreatePen(PS_SOLID, Width, RGB(R, G, B));
  NewBrush := CreateSolidBrush(RGB(R, G, B));
  OldPen := SelectObject(m_hBitmapDC, NewPen);
  OldBrush := SelectObject(m_hBitmapDC, NewBrush);
  Rectangle(m_hBitmapDC, 0, 0, Width, Height);
  SelectObject(m_hBitmapDC, OldBrush);
  SelectObject(m_hBitmapDC, OldPen);
  DeleteObject(NewBrush);
  DeleteObject(NewPen);
end;

//-------------------------------------------------------------------------------------------------------
// Draw a line in bitmap
//-------------------------------------------------------------------------------------------------------
procedure CDisplay.DrawLine(X1, Y1, X2, Y2, Width: Integer; R, G, B: Byte);
var
  OldPen, NewPen: HPEN;
begin
  NewPen := CreatePen(PS_SOLID, Width, RGB(R, G, B));
  OldPen := SelectObject(m_hBitmapDC, NewPen);
  MoveToEx(m_hBitmapDC, X1, Y1, nil);
  LineTo(m_hBitmapDC, X2, Y2);
  SelectObject(m_hBitmapDC, OldPen);
  DeleteObject(NewPen);
end;

//-------------------------------------------------------------------------------------------------------
// Backsurface presentation
//-------------------------------------------------------------------------------------------------------
procedure CDisplay.Draw(WindowHandle: HWND);
var
  DC: HDC;
begin
  QueryPerformanceCounter(m_nActualTime);
  while ((m_nActualTime - m_nStartTime) / m_nFrequency < 0.01) do
    QueryPerformanceCounter(m_nActualTime);
  m_nStartTime := m_nActualTime;

  DC := GetDC(WindowHandle);
  BitBlt(DC, 0, 0, Width, Height, m_hBitmapDC, 0, 0, SRCCOPY);
  ReleaseDC(WindowHandle, DC);
end;

end.

