/*
 * PhysicsTest
 *
 * Written by Soothsayer in January 2010
 *
 * Website: http://www.cppanddelphi.atw.hu
 * Contact: cppanddelphi@gmail.com
 */
unit _window;

interface

uses 
  Messages, MMSystem, Windows;
var
  m_bActive: Boolean;
  m_bReplay: Boolean = False;
  m_bKeyBuffer: array[0..255] of Boolean;
type
//-------------------------------------------------------------------------------------------------------
// Class window
//-------------------------------------------------------------------------------------------------------
  CWindow = class

  private
    procedure Error(ErrorString: PChar);
  public
    Handle:                 HWND;
    Instance:               HINST;

    function DoEvents(): Boolean;
    constructor Create(AppName: PChar; WindowWidth, WindowHeight: Integer);
    destructor Destroy();
  end;

implementation

//-------------------------------------------------------------------------------------------------------
// Perform windows messages
//-------------------------------------------------------------------------------------------------------
function WinProc(hWnd: HWND; Messages, wParam, lParam: Cardinal): Cardinal; stdcall;
begin
  case Messages of

    MM_MCINOTIFY:
    begin
      if (wParam and MCI_NOTIFY_SUCCESSFUL) <> 0 then 
        m_bReplay := True;
      Result := 0; Exit;
    end;

    WM_ACTIVATEAPP:
    begin
      m_bActive := wParam <> 0;
      Result := 0; Exit;
    end;

    WM_KEYDOWN:
    begin
      m_bKeyBuffer[LOWORD(wParam)] := True;
      Result := 0; Exit;
    end;

    WM_KEYUP:
    begin
      m_bKeyBuffer[LOWORD(wParam)] := False;
      Result := 0; Exit;
    end;

    WM_DESTROY:
    begin
      PostMessage(hWnd, WM_QUIT, 0, 0);
      Result := 0; Exit;
    end;

  else
    Result := DefWindowProc(hWnd, Messages, wParam, lParam);
  end;
end;

//-------------------------------------------------------------------------------------------------------
// Write any errors to screen
//-------------------------------------------------------------------------------------------------------
procedure CWindow.Error(ErrorString: PChar);
begin
  MessageBox(0, ErrorString, 'Error', MB_OK);
  PostQuitMessage(0);
end;

//-------------------------------------------------------------------------------------------------------
// Process windows messages
//-------------------------------------------------------------------------------------------------------
function CWindow.DoEvents(): Boolean;
var
  Messages: MSG;
begin
  if PeekMessage(Messages, 0, 0, 0, PM_NOREMOVE)
  then begin
    if GetMessage(Messages, 0, 0, 0)
    then begin
      TranslateMessage(Messages);
      DispatchMessage(Messages);
    end
    else 
    begin
      Result := False; Exit;
    end;
  end
  else 
  begin
    if m_bActive then 
      Result := True
    else 
      WaitMessage;
  end;

  Result := True;
end;

//-------------------------------------------------------------------------------------------------------
// Create window class
//-------------------------------------------------------------------------------------------------------
constructor CWindow.Create(AppName: PChar; WindowWidth, WindowHeight: Integer);
var
  wc: WNDCLASS;
  XFrame, YFrame, XSize, YSize: Integer;
begin
  Instance := GetModuleHandle(0);
  if (not GetClassInfo(Instance, 'CWindow', wc))
  then begin
    wc.style         := CS_HREDRAW or CS_VREDRAW;
    wc.lpfnWndProc   := @WinProc;
    wc.cbClsExtra    := 0;
    wc.cbWndExtra    := 0;
    wc.hInstance     := Instance;
    wc.hIcon         := LoadIcon(0, IDI_WINLOGO);
    wc.hCursor       := LoadCursor(0, IDC_ARROW);
    wc.hbrBackground := 0;
    wc.lpszMenuName  := 0;
    wc.lpszClassName := 'CWindow';
  end;
  if (RegisterClass(wc) = 0) then
    Error('Couldn`t register windows class');

  XFrame := 2 * GetSystemMetrics(SM_CXFIXEDFRAME);
  YFrame := 2 * GetSystemMetrics(SM_CYFIXEDFRAME) + GetSystemMetrics(SM_CYCAPTION);
  XSize  := WindowWidth + XFrame;
  YSize  := WindowHeight + YFrame;

  Handle := CreateWindow('CWindow', AppName,
                         WS_OVERLAPPED or WS_VISIBLE or WS_SYSMENU or WS_CAPTION or WS_MINIMIZEBOX,
                         (GetSystemMetrics(SM_CXSCREEN) - XSize) shr 1,
                         (GetSystemMetrics(SM_CYSCREEN) - YSize) shr 1,
                         WindowWidth + XFrame,
                         WindowHeight + YFrame,
                         0, 0, Instance, 0);
  if (Handle = 0) then
    Error('Couldn`t create window');

  ShowWindow(Handle, SW_SHOW);
  SetFocus(Handle);
end;

//-------------------------------------------------------------------------------------------------------
// Destroy window class
//-------------------------------------------------------------------------------------------------------
destructor CWindow.Destroy();
begin
  DestroyWindow(Handle);
  UnregisterClass('CWindow', Instance);
end;

end.
