/*
 * PhysicsTest
 *
 * Written by Soothsayer in January 2010
 *
 * Website: http://www.cppanddelphi.atw.hu
 * Contact: cppanddelphi@gmail.com
 */
uses
  Windows, _display, _window, TVector, TPhysics;
const
  AppName = 'PhysicsTest';
var
  Window: CWindow;
  Bitmap: CDisplay;
  World: CWorld;
  i, j, k: Integer;
  S: CVector;
  V: CVector;
  V1, V2, V3, V4: CVector;
  A: CArbiter;

procedure PopulateWorld;
var
  B: CBody;
  V1, V2, offset: CVector;
  i, j, n: Integer;
begin
  World := CWorld.Create(CreateV(0, -10), 20);
  B := CBody.Create(CreateV(1000, 20), 3.4e+38);
  B.Friction := 1.2;
  B.Position := CreateV(0, -50);
  B.Rotation := 0;
  World.Add(B);

  n := 6;
  for i := 1 to n do  
  begin
    offset := CreateV(-i * 60 / 2.0, (n - i) * 60 + 104);
    for j := 0 to i - 1 do
    begin
      B := CBody.Create(CreateV(10, 40), 1);
      B.Position := AddV(CreateV(j * 60, -125), offset);
      World.Add(B);

      B := CBody.Create(CreateV(10, 40),1);
      B.Position := AddV(CreateV(j * 60, -100), offset);
      B.Rotation := pi / 2;
      World.Add(B);

      if (j = (i - 1)) then continue;
      B := CBody.Create(CreateV(10, 40),1);
      B.Position := AddV(CreateV(j * 60 + 30, -90), offset);
      B.Rotation := pi / 2;
      World.Add(B);
    end;
    B := CBody.Create(CreateV(10, 40),1);
    B.Position := AddV(CreateV(-15, -75), offset);
    World.Add(B);

    B := CBody.Create(CreateV(10, 40),1);
    B.Position := AddV(CreateV((i - 1) * 60 + 15, -75), offset);
    World.Add(B);
  end;

  B := CBody.Create(CreateV(10,10), 0.5);
  B.Friction := 1.2;
  B.Position := CreateV(200, 200);
  World.Add(B);
end;

begin
  Window := CWindow.Create(AppName, 640, 400);
  Bitmap := CDisplay.Create({Window, 640, 400});
  Bitmap.Init;

  PopulateWorld;
  while (Window.DoEvents) and (not m_bKeyBuffer[VK_ESCAPE]) do
  begin
    Bitmap.Clear(80, 130, 80);
    if (m_bKeyBuffer[VK_SPACE])
    then begin
      World.FBodies[High(World.FBodies)].Position := CreateV(250 - Random(500), 250);
      World.FBodies[High(World.FBodies)].Rotation := Random(2) - 4;
      World.FBodies[High(World.FBodies)].Velocity := MultiplyV(CreateV(-World.FBodies[High(World.FBodies)].Position.x, -250), 0.5);
      World.FBodies[High(World.FBodies)].AngularVelocity := Random(2) - 4;
    end;

    for i := 0 to High(World.FBodies) do
    begin
      for j := 0 to high(World.FBodies[I].poly) - 1 do
        Bitmap.DrawLine(Round(World.FBodies[I].poly[j].x * 0.9) + 320, -Round(World.FBodies[I].poly[j].y* 0.9) + 320,
                        Round(World.FBodies[I].poly[j + 1].x* 0.9) + 320, -Round(World.FBodies[I].poly[j + 1].y* 0.9) + 320, 2, 0, 0, 0);
    end;

    for I := 0 to ArbiterCount do
    begin
      A := ArbiterBuckets[ArbiterList[i].Body1, ArbiterList[i].Body2];
      for k := 0 to A.FNumContacts - 1 do
        Bitmap.DrawLine(Round(A.FContacts[k].Position.x * 0.9) + 320, -Round(A.FContacts[k].Position.y * 0.9) + 320,
                        Round(A.FContacts[k].Position.x * 0.9) + 320, -Round(A.FContacts[k].Position.y * 0.9) + 320, 3, 255, 0, 0);
    end;

    World.Step(0.066);

    Bitmap.Draw(Window.Handle);
  end;

  Bitmap.Destroy;
  Window.Destroy;
end.

